@@ -50,9 +50,9 @@ struct InfoScreen_ {
 };
 }*/
 
-static const char* InfoScreenFunctions[] = {"Search ", "Filter ", "Refresh", "Done   ", NULL};
+static const char* const InfoScreenFunctions[] = {"Search ", "Filter ", "Refresh", "Done   ", NULL};
 
-static const char* InfoScreenKeys[] = {"F3", "F4", "F5", "Esc"};
+static const char* const InfoScreenKeys[] = {"F3", "F4", "F5", "Esc"};
 
 static int InfoScreenEvents[] = {KEY_F(3), KEY_F(4), KEY_F(5), 27};
 
@@ -83,6 +83,7 @@ void InfoScreen_drawTitled(InfoScreen* this, char* fmt, ...) {
    wmove(stdscr, 0, 0);
    vw_printw(stdscr, fmt, ap);
    attrset(CRT_colors[DEFAULT_COLOR]);
+   this->display->needsRedraw = true;
    Panel_draw(this->display, true);
    IncSet_drawBar(this->inc);
    va_end(ap);
@@ -116,6 +117,7 @@ void InfoScreen_run(InfoScreen* this) {
 
       if (this->inc->active)
          move(LINES-1, CRT_cursorX);
+      set_escdelay(25);
       int ch = getch();
       
       if (ch == ERR) {
