@@ -37,7 +37,7 @@ typedef struct Meter_ Meter;
 typedef void(*Meter_Init)(Meter*);
 typedef void(*Meter_Done)(Meter*);
 typedef void(*Meter_UpdateMode)(Meter*, int);
-typedef void(*Meter_SetValues)(Meter*, char*, int);
+typedef void(*Meter_UpdateValues)(Meter*, char*, int);
 typedef void(*Meter_Draw)(Meter*, int, int, int);
 
 typedef struct MeterClass_ {
@@ -46,7 +46,7 @@ typedef struct MeterClass_ {
    const Meter_Done done;
    const Meter_UpdateMode updateMode;
    const Meter_Draw draw;
-   const Meter_SetValues setValues;
+   const Meter_UpdateValues updateValues;
    const int defaultMode;
    const double total;
    const int* attributes;
@@ -66,7 +66,8 @@ typedef struct MeterClass_ {
 #define Meter_updateMode(this_, m_)    As_Meter(this_)->updateMode((Meter*)(this_), m_)
 #define Meter_drawFn(this_)            As_Meter(this_)->draw
 #define Meter_doneFn(this_)            As_Meter(this_)->done
-#define Meter_setValues(this_, c_, i_) As_Meter(this_)->setValues((Meter*)(this_), c_, i_)
+#define Meter_updateValues(this_, buf_, sz_) \
+                                       As_Meter(this_)->updateValues((Meter*)(this_), buf_, sz_)
 #define Meter_defaultMode(this_)       As_Meter(this_)->defaultMode
 #define Meter_getItems(this_)          As_Meter(this_)->curItems
 #define Meter_setItems(this_, n_)      As_Meter(this_)->curItems = (n_)
@@ -132,12 +133,8 @@ Meter* Meter_new(struct ProcessList_* pl, int param, MeterClass* type) {
    this->h = 1;
    this->param = param;
    this->pl = pl;
-   char maxItems = type->maxItems;
-   if (maxItems == 0) {
-      maxItems = 1;
-   }
-   type->curItems = maxItems;
-   this->values = xCalloc(maxItems, sizeof(double));
+   type->curItems = type->maxItems;
+   this->values = xCalloc(type->maxItems, sizeof(double));
    this->total = type->total;
    this->caption = xStrdup(type->caption);
    if (Meter_initFn(this))
@@ -155,7 +152,7 @@ int Meter_humanUnit(char* buffer, unsigned long int value, int size) {
       if (value / 1024 < powi)
          break;
 
-      if (prefix[1] == 0)
+      if (prefix[1] == '\0')
          break;
 
       powi *= 1024;
@@ -184,8 +181,7 @@ void Meter_delete(Object* cast) {
    if (Meter_doneFn(this)) {
       Meter_done(this);
    }
-   if (this->drawData)
-      free(this->drawData);
+   free(this->drawData);
    free(this->caption);
    free(this->values);
    free(this);
@@ -216,8 +212,7 @@ void Meter_setMode(Meter* this, int modeIndex) {
          Meter_updateMode(this, modeIndex);
    } else {
       assert(modeIndex >= 1);
-      if (this->drawData)
-         free(this->drawData);
+      free(this->drawData);
       this->drawData = NULL;
 
       MeterMode* mode = Meter_modes[modeIndex];
@@ -230,16 +225,16 @@ void Meter_setMode(Meter* this, int modeIndex) {
 ListItem* Meter_toListItem(Meter* this, bool moving) {
    char mode[21];
    if (this->mode)
-      snprintf(mode, 20, " [%s]", Meter_modes[this->mode]->uiName);
+      xSnprintf(mode, 20, " [%s]", Meter_modes[this->mode]->uiName);
    else
       mode[0] = '\0';
    char number[11];
    if (this->param > 0)
-      snprintf(number, 10, " %d", this->param);
+      xSnprintf(number, 10, " %d", this->param);
    else
       number[0] = '\0';
    char buffer[51];
-   snprintf(buffer, 50, "%s%s%s", Meter_uiName(this), number, mode);
+   xSnprintf(buffer, 50, "%s%s%s", Meter_uiName(this), number, mode);
    ListItem* li = ListItem_new(buffer, 0);
    li->moving = moving;
    return li;
@@ -249,7 +244,7 @@ ListItem* Meter_toListItem(Meter* this, bool moving) {
 
 static void TextMeterMode_draw(Meter* this, int x, int y, int w) {
    char buffer[METER_BUFFER_LEN];
-   Meter_setValues(this, buffer, METER_BUFFER_LEN - 1);
+   Meter_updateValues(this, buffer, METER_BUFFER_LEN - 1);
    (void) w;
 
    attrset(CRT_colors[METER_TEXT]);
@@ -265,11 +260,11 @@ static void TextMeterMode_draw(Meter* this, int x, int y, int w) {
 
 /* ---------- BarMeterMode ---------- */
 
-static char BarMeterMode_characters[] = "|#*@$%&.";
+static const char BarMeterMode_characters[] = "|#*@$%&.";
 
 static void BarMeterMode_draw(Meter* this, int x, int y, int w) {
    char buffer[METER_BUFFER_LEN];
-   Meter_setValues(this, buffer, METER_BUFFER_LEN - 1);
+   Meter_updateValues(this, buffer, METER_BUFFER_LEN - 1);
 
    w -= 2;
    attrset(CRT_colors[METER_TEXT]);
@@ -291,11 +286,8 @@ static void BarMeterMode_draw(Meter* this, int x, int y, int w) {
    char bar[w + 1];
    
    int blockSizes[10];
-   for (int i = 0; i < w; i++)
-      bar[i] = ' ';
 
-   const size_t barOffset = w - MIN((int)strlen(buffer), w);
-   snprintf(bar + barOffset, w - barOffset + 1, "%s", buffer);
+   xSnprintf(bar, w + 1, "%*s", w, buffer);
 
    // First draw in the bar[] buffer...
    int offset = 0;
@@ -344,7 +336,7 @@ static void BarMeterMode_draw(Meter* this, int x, int y, int w) {
 #ifdef HAVE_LIBNCURSESW
 
 #define PIXPERROW_UTF8 4
-static const char* GraphMeterMode_dotsUtf8[] = {
+static const char* const GraphMeterMode_dotsUtf8[] = {
    /*00*/" ", /*01*/"⢀", /*02*/"⢠", /*03*/"⢰", /*04*/ "⢸",
    /*10*/"⡀", /*11*/"⣀", /*12*/"⣠", /*13*/"⣰", /*14*/ "⣸",
    /*20*/"⡄", /*21*/"⣄", /*22*/"⣤", /*23*/"⣴", /*24*/ "⣼",
@@ -355,13 +347,13 @@ static const char* GraphMeterMode_dotsUtf8[] = {
 #endif
 
 #define PIXPERROW_ASCII 2
-static const char* GraphMeterMode_dotsAscii[] = {
+static const char* const GraphMeterMode_dotsAscii[] = {
    /*00*/" ", /*01*/".", /*02*/":",
    /*10*/".", /*11*/".", /*12*/":",
    /*20*/":", /*21*/":", /*22*/":"
 };
 
-static const char** GraphMeterMode_dots;
+static const char* const* GraphMeterMode_dots;
 static int GraphMeterMode_pixPerRow;
 
 static void GraphMeterMode_draw(Meter* this, int x, int y, int w) {
@@ -397,7 +389,7 @@ static void GraphMeterMode_draw(Meter* this, int x, int y, int w) {
          data->values[i] = data->values[i+1];
    
       char buffer[nValues];
-      Meter_setValues(this, buffer, nValues - 1);
+      Meter_updateValues(this, buffer, nValues - 1);
    
       double value = 0.0;
       int items = Meter_getItems(this);
@@ -414,8 +406,8 @@ static void GraphMeterMode_draw(Meter* this, int x, int y, int w) {
    }
    for (; i < nValues; i+=2, k++) {
       int pix = GraphMeterMode_pixPerRow * GRAPH_HEIGHT;
-      int v1 = CLAMP(data->values[i] * pix, 1, pix);
-      int v2 = CLAMP(data->values[i+1] * pix, 1, pix);
+      int v1 = CLAMP((int) lround(data->values[i] * pix), 1, pix);
+      int v2 = CLAMP((int) lround(data->values[i+1] * pix), 1, pix);
 
       int colorIdx = GRAPH_1;
       for (int line = 0; line < GRAPH_HEIGHT; line++) {
@@ -432,7 +424,7 @@ static void GraphMeterMode_draw(Meter* this, int x, int y, int w) {
 
 /* ---------- LEDMeterMode ---------- */
 
-static const char* LEDMeterMode_digitsAscii[] = {
+static const char* const LEDMeterMode_digitsAscii[] = {
    " __ ","    "," __ "," __ ","    "," __ "," __ "," __ "," __ "," __ ",
    "|  |","   |"," __|"," __|","|__|","|__ ","|__ ","   |","|__|","|__|",
    "|__|","   |","|__ "," __|","   |"," __|","|__|","   |","|__|"," __|"
@@ -440,7 +432,7 @@ static const char* LEDMeterMode_digitsAscii[] = {
 
 #ifdef HAVE_LIBNCURSESW
 
-static const char* LEDMeterMode_digitsUtf8[] = {
+static const char* const LEDMeterMode_digitsUtf8[] = {
    "┌──┐","  ┐ ","╶──┐","╶──┐","╷  ╷","┌──╴","┌──╴","╶──┐","┌──┐","┌──┐",
    "│  │","  │ ","┌──┘"," ──┤","└──┤","└──┐","├──┐","   │","├──┤","└──┤",
    "└──┘","  ╵ ","└──╴","╶──┘","   ╵","╶──┘","└──┘","   ╵","└──┘"," ──┘"
@@ -448,7 +440,7 @@ static const char* LEDMeterMode_digitsUtf8[] = {
 
 #endif
 
-static const char** LEDMeterMode_digits;
+static const char* const* LEDMeterMode_digits;
 
 static void LEDMeterMode_drawDigit(int x, int y, int n) {
    for (int i = 0; i < 3; i++)
@@ -466,7 +458,7 @@ static void LEDMeterMode_draw(Meter* this, int x, int y, int w) {
       LEDMeterMode_digits = LEDMeterMode_digitsAscii;
 
    char buffer[METER_BUFFER_LEN];
-   Meter_setValues(this, buffer, METER_BUFFER_LEN - 1);
+   Meter_updateValues(this, buffer, METER_BUFFER_LEN - 1);
    
    RichString_begin(out);
    Meter_displayBuffer(this, buffer, &out);
@@ -529,7 +521,7 @@ MeterMode* Meter_modes[] = {
 
 /* Blank meter */
 
-static void BlankMeter_setValues(Meter* this, char* buffer, int size) {
+static void BlankMeter_updateValues(Meter* this, char* buffer, int size) {
    (void) this; (void) buffer; (void) size;
 }
 
@@ -548,8 +540,9 @@ MeterClass BlankMeter_class = {
       .delete = Meter_delete,
       .display = BlankMeter_display,
    },
-   .setValues = BlankMeter_setValues,
+   .updateValues = BlankMeter_updateValues,
    .defaultMode = TEXT_METERMODE,
+   .maxItems = 0,
    .total = 100.0,
    .attributes = BlankMeter_attributes,
    .name = "Blank",
