@@ -24,7 +24,7 @@ int SwapMeter_attributes[] = {
    SWAP
 };
 
-static void SwapMeter_setValues(Meter* this, char* buffer, int size) {
+static void SwapMeter_updateValues(Meter* this, char* buffer, int size) {
    int written;
    Platform_setSwapValues(this);
 
@@ -54,8 +54,9 @@ MeterClass SwapMeter_class = {
       .delete = Meter_delete,
       .display = SwapMeter_display,
    },
-   .setValues = SwapMeter_setValues, 
+   .updateValues = SwapMeter_updateValues,
    .defaultMode = BAR_METERMODE,
+   .maxItems = 1,
    .total = 100.0,
    .attributes = SwapMeter_attributes,
    .name = "Swap",
